# -*- coding: utf-8 -*-
__author__ = 'ggegiya1'
import tempfile
import os
import requests


class ArtifactoryHelper(object):
    """
    Helper class to facilitate interacting with Artifactory
    """

    build_server_url = 'http://artifactory.ypdev.ca/artifactory'
    repository = ''
    group_id = ''

    def __init__(self, repository, group_id):
        """
        Constructor method
        :param repository: (str) Name of the repo in Artifactory (ie: libs-release-local)
        :param group_id: (str) Name of the group in Artifactory (ie: com.ypg.yid)
        """

        self.repository = repository
        self.group_id = group_id

    def get_versions(self, artifact_id):
        """
        Retrieve all the available versions for the specified artifact
        :param artifact_id: (str) name of the artifact in Artifactory
        :return: (list) A list of versions sorted from latest to oldest
        """

        versions_url = "{url}/api/search/versions?g={group_id}&a={artifact_id}&repos={repository}"
        versions_url = versions_url.format(url=self.build_server_url, group_id=self.group_id,
                                           artifact_id=artifact_id, repository=self.repository)
        resp = requests.get(versions_url)
        versions = resp.json()
        if hasattr(versions, "__iter__"):
            return map(lambda x: x["version"], versions["results"])
        return []

    def get_artifact(self, artifact_id, version, criteria=None):
        """
        Fetches the artifact from Artifactory and write the file to disk.  A criteria can be added to help
        :param artifact_id: (str) name of the artifact in Artifactory
        :param version: (str) numeric representation of the artifact version
        :param criteria: (str) a filter criteria for choosing the artifact to download (ex: .jar, -QA.msi)
        :return: (list)  a list of successfully downloaded files
        """

        search_url = "{url}/api/search/gavc?a={artifact_id}&v={version}"
        search_url = search_url.format(url=self.build_server_url,
                                           artifact_id=artifact_id, version=version)
        temp = requests.get(search_url, stream=True)

        resp = temp.json()['results']

        to_download = []
        downloaded = []
        if criteria:
            for url in resp:
                if criteria in url['uri']:
                    to_download.append(url['uri'])
        else:
            for url in resp:
                to_download.append(url['uri'])

        for download in to_download:
            filename = download.split('/')[-1]
            file = os.path.join(tempfile.gettempdir(), filename)
            download_url = self.build_server_url+download.split('storage')[1]
            headers = {'Accept-Encoding': 'identity, deflate, compress', 'Accept': '*/*',
                       'User-Agent': 'python-requests/1.2.0'}
            if '.msi' not in filename:
                temp = requests.get(download_url, stream=True, headers=headers)
                with open(file, 'wb') as f:
                    f.write(temp.raw.data)

                if os.path.exists(file):
                    downloaded.append(file)

        return downloaded

    def cleanup_downloads(self, downloads):
        """
        Removes the specified files
        :param downloads: (list) a list of files to delete
        :return: (bool) If files were removed or not
        """
        for file in downloads:
            try:
                os.remove(file)
            except OSError:
                return False
        return True